//Write new shader file with all point lights in the server
function writeManyPointLights()
{
	if(!isObject(ServerConnection))
		return;

	%write = new FileObject();
	%write.openForWrite("config/manyPointLights_frag.glsl");

	//Copy the start of the shader file
	%read = new FileObject();
	%read.openForRead("Add-Ons/Client_ManyPointLights/_1.glsl");

	while(!%read.isEOF())
		%write.writeLine(%read.readLine());

	%read.close();

	//Load the repeating part of the shader file
	%read.openForRead("Add-Ons/Client_ManyPointLights/_2.glsl");
	%lineCount = 0;

	while(!%read.isEOF())
	{
		%line[%lineCount] = %read.readLine();
		%lineCount++;
	}

	%read.close();

	//Write all the point lights
	%count = ServerConnection.getCount();
	%lightCount = 0;

	for(%i = 0; %i < %count; %i++)
	{
		%obj = ServerConnection.getObject(%i);

		if(%obj.getClassName() $= "FxLight")
		{
			//Get point light data
			%pos = %obj.getWorldBoxCenter();
			%color = vectorScale(%obj.getDatablock().color, %obj.getDatablock().brightness);
			%radius = %obj.getDatablock().radius * 1.5;

			%pos = "vec3("
				@ getWord(%pos, 0) @ "f, "
				@ getWord(%pos, 1) @ "f, "
				@ getWord(%pos, 2) @ "f)";

			%color = "vec3("
				@ getWord(%color, 0) @ "f, "
				@ getWord(%color, 1) @ "f, "
				@ getWord(%color, 2) @ "f)";

			%radius = %radius @ "f";
			
			//Write point light to shader
			for(%j = 0; %j < %lineCount; %j++)
			{
				%l = %line[%j];
				%l = strReplace(%l, "$POSITION$", %pos);
				%l = strReplace(%l, "$COLOR$", %color);
				%l = strReplace(%l, "$RADIUS$", %radius);
				%write.writeLine(%l);
			}

			%lightCount++;
		}
	}

	//Copy the end of the shader file
	%read.openForRead("Add-Ons/Client_ManyPointLights/_3.glsl");

	while(!%read.isEOF())
		%write.writeLine(%read.readLine());

	%read.close();
	%write.close();

	//Copy the other shader file
	%read.openForRead("Add-Ons/Client_ManyPointLights/manyPointLights_vert.glsl");
	%write.openForWrite("config/manyPointLights_vert.glsl");

	while(!%read.isEOF())
		%write.writeLine(%read.readLine());

	%read.delete();
	%write.delete();

	echo("Wrote new shader with " @ %lightCount @ " point lights.");
	echo("1. Copy files config/manyPointLights_*.glsl to shaders/ directory");
	echo("2. Type applyManyPointLights();");
}

//Change the shader to the one we generated earlier
function applyManyPointLights()
{
	$Shader::ShaderName = "manyPointLights";
	initializeShaderAssets();
	regenerateShadowMapFBOs();
	flushVBOCache();
}
